const enrich = require("../main");
const expect = require('chai').expect;
const raw = require("fs").readFileSync(__dirname + "/address.txt", "UTF-8");

// サンプルデータ 14.7万件に対して実行し、実行時間の計測、結果の記録を行います。

let output = "表記,都道府県,市区町村,区,町名,丁目,番地,号,種別,説明\n";

describe('imi-enrichment-address', function() {

  describe('stat', function() {

    const limit = {
      "号": 0,
      "番地": 0,
      "丁目": 0,
      "町名": 0,
      "区": 0,
      "市区町村": 0,
      "都道府県": 0,
      "候補なし": 0,
      "候補複数": 0
    };

    const found = {
      "号": 0,
      "番地": 0,
      "丁目": 0,
      "町名": 0,
      "区": 0,
      "市区町村": 0,
      "都道府県": 0
    };

    const appear = {
      "町名": 0,
      "区": 0,
      "市区町村": 0,
      "都道府県": 0
    };

    const score = {
      start: new Date().getTime(),
      total: 0
    };

    const bad = [];

    raw.split("\n").filter((a, i) => a.length > 0).forEach(address => {
      score.total++;
      const input = {
        "@context": "https://imi.go.jp/ns/core/context.jsonld",
        "@type": "住所型",
        "表記": address
      };
      const result = enrich(input);

      // どの深さまで分かち書きできたか
      if (result["号"] !== undefined) limit["号"]++;
      else if (result["番地"] !== undefined) limit["番地"]++;
      else if (result["丁目"] !== undefined) limit["丁目"]++;
      else if (result["町名"] !== undefined) limit["町名"]++;
      else if (result["区"] !== undefined) limit["区"]++;
      else if (result["市区町村"] !== undefined) limit["市区町村"]++;
      else if (result["都道府県"] !== undefined) limit["都道府県"]++;
      else if (result["メタデータ"]["説明"].indexOf("複数") !== -1) {
        limit["候補複数"]++;
        bad.push(result);
      } else {
        limit["候補なし"]++;
        bad.push(result);
      }

      // プロパティごとの発生件数
      Object.keys(found).filter(key => result[key] !== undefined).forEach(key => {
        found[key]++;
      });

      // 補完された値が元の表記に含まれるかどうか
      Object.keys(appear).filter(key => result[key] !== undefined).forEach(key => {
        if (address.replace(/[ 　]/g, "").indexOf(result[key]) === -1) {
          appear[key]++;
        }
      });

      const row = [
        result["表記"] || "",
        result["都道府県"] || "",
        result["市区町村"] || "",
        result["区"] || "",
        result["町名"] || "",
        result["丁目"] || "",
        result["番地"] || "",
        result["号"] || "",
        result["メタデータ"]["種別"] || "",
        result["メタデータ"]["説明"] || ""
      ];

      output += row.join(",") + "\n";
    });

    require("fs").writeFileSync(__dirname + "/output.csv", '\uFEFF' + output, "UTF-8");



    score.end = new Date().getTime();
    score.duration = score.end - score.start;
    score.average = score.duration / score.total;

    describe("全体", () => {
      it(`入力件数: ${score.total} (${(score.total/score.total*100).toFixed(2)}%)`, () => {
        expect(score.total).to.equal(147756);
      });
      it(`実行時間: ${score.duration} [msec]`, () => {
        expect(score.duration).to.be.below(60 * 1000);
      });
      it(`10,000件あたり: ${(score.duration / score.total * 10000).toFixed(2)} [msec]`, () => {
        expect(score.duration / score.total * 10000).to.be.below(2000);
      });

    });

    describe("付与された最深プロパティ", () => {
      const a = {
        '都道府県': 132,
        '市区町村': 3994,
        '区': 884,
        '町名': 13936,
        '丁目': 157,
        '番地': 25711,
        '号': 102820,
        '候補複数': 17,
        '候補なし': 105
      };
      Object.keys(a).forEach(key => {
        it(`${key}: ${limit[key]} (${(limit[key]/score.total*100).toFixed(2)}%)`, () => {
          expect(limit[key]).to.equal(a[key]);
        });
      });
    });

    describe("プロパティごとの付与数", () => {
      const a = {
        '都道府県': 147634,
        '市区町村': 147502,
        '区': 32661,
        '町名': 142624,
        '丁目': 70505,
        '番地': 128531,
        '号': 102820
      };
      Object.keys(a).forEach(key => {
        it(`${key}: ${found[key]} (${(found[key]/score.total*100).toFixed(2)}%)`, () => {
          expect(found[key]).to.equal(a[key]);
        });
      });
    });

    describe("プロパティごとの補完数", () => {
      const a = {
        '都道府県': 10403,
        '市区町村': 116,
        '区': 209,
        '町名': 0
      };
      Object.keys(a).forEach(key => {
        it(`${key}: ${appear[key]} (${(appear[key]/found[key]*100).toFixed(2)}%)`, () => {
          expect(appear[key]).to.equal(a[key]);
        });
      });
    });

    describe("異常系", () => {
      describe("国内住所でないと判定されたもの", () => {
        bad.filter(a => a["メタデータ"]["説明"].indexOf("日本国内の住所") !== -1).forEach(a => {
          it(a["表記"], () => {
            expect(1).to.equal(1);
          });
        });
      });
      describe("候補がなかった住所", () => {
        bad.filter(a => a["メタデータ"]["説明"].indexOf("都道府県を特定") !== -1).forEach(a => {
          it(a["表記"], () => {
            expect(1).to.equal(1);
          });
        });
      });
      describe("候補が複数あった住所", () => {
        bad.filter(a => a["メタデータ"]["説明"].indexOf("複数") !== -1).forEach(a => {
          it(`${a["表記"]} ${a["メタデータ"]["説明"].split(" ").pop()}`, () => {
            expect(1).to.equal(1);
          });
        });
      });
    });
  });
});
